@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 7/28/2019.
-//  Copyright (c) 2019-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2019-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -39,6 +39,7 @@
 #include <QAbstractTextDocumentLayout>
 #include <QPalette>
 #include <QApplication>
+#include <QDateTime>
 #include <QDebug>
 #include <cmath>
 
@@ -655,6 +656,14 @@ QString attributedStringForByteCount(uint64_t bytes, double total, QTextCharForm
 	return byteString;
 }
 
+QString slimDateline(void)
+{
+    QDateTime dateTime = QDateTime::currentDateTime();
+    QString dateTimeString = dateTime.toString("M/d/yy, h:mm:ss AP");        // format: 3/28/20, 8:03:09 PM
+    
+    return QString("# %1").arg(dateTimeString);
+}
+
 // Running a panel to obtain numbers from the user
 // The goal here is to avoid a proliferation of dumb forms, by programmatically generating the UI
 
