@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 7/28/2019.
-//  Copyright (c) 2019-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2019-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -21,6 +21,7 @@
 #define QTSLIMEXTRAS_H
 
 #include <QObject>
+#include <QString>
 #include <QWidget>
 #include <QColor>
 #include <QRect>
@@ -43,6 +44,14 @@
 
 class QPaintEvent;
 
+// legend positions for QtSLiMGraphView
+typedef enum {
+    kUnconfigured = -1,
+    kTopLeft = 0,
+    kTopRight,
+    kBottomLeft,
+    kBottomRight
+} QtSLiM_LegendPosition;
 
 void QtSLiMFrameRect(const QRect &p_rect, const QColor &p_color, QPainter &p_painter);
 void QtSLiMFrameRect(const QRectF &p_rect, const QColor &p_color, QPainter &p_painter, double p_lineWidth);
@@ -120,6 +129,9 @@ QColor slimColorForFraction(double fraction);
 QString stringForByteCount(uint64_t bytes);
 QString attributedStringForByteCount(uint64_t bytes, double total, QTextCharFormat &format);
 
+// Nicely formatted dateline for output
+QString slimDateline(void);
+
 // Running a panel to obtain numbers from the user
 QStringList QtSLiMRunLineEditArrayDialog(QWidget *p_parent, QString title, QStringList captions, QStringList values);
 
