@@ -3,7 +3,7 @@
 //  SLiM
 //
 //  Created by Ben Haller on 2/28/2022.
-//  Copyright (c) 2022-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2022-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -72,6 +72,7 @@ private:
 	std::vector<SLiMEidosBlock*> scheduled_interaction_deregs_;						// NOT OWNED: interaction() callbacks in script_blocks_ that are scheduled for deregistration
 	
 	// a cache of the last tick for the simulation, for speed
+	bool tick_ranges_available_ = false;											// false until tick ranges have been determined, after initialize() callbacks
 	bool last_script_block_tick_cached_ = false;
 	slim_tick_t last_script_block_tick_;											// the last tick in which a bounded script block is scheduled to run
 	
@@ -239,13 +240,13 @@ public:
 	
 	// Checking for species identity; these return nullptr if the objects do not all belong to the same species
 	// Calls to these methods, and other such species checks, should be labeled SPECIES CONSISTENCY CHECK to make them easier to find
-	static Species *SpeciesForIndividualsVector(Individual **individuals, int value_count);
+	static Species *SpeciesForIndividualsVector(const Individual * const *individuals, int value_count);
 	static Species *SpeciesForIndividuals(EidosValue *value);
 	
-	static Species *SpeciesForGenomesVector(Genome **genomes, int value_count);
+	static Species *SpeciesForGenomesVector(const Genome * const *genomes, int value_count);
 	static Species *SpeciesForGenomes(EidosValue *value);
 	
-	static Species *SpeciesForMutationsVector(Mutation **mutations, int value_count);
+	static Species *SpeciesForMutationsVector(const Mutation * const *mutations, int value_count);
 	static Species *SpeciesForMutations(EidosValue *value);
 	
 	// Running ticks
@@ -260,6 +261,8 @@ public:
 	bool _RunOneTickWF(void);														// called by _RunOneTick() to run a tick (WF models)
 	bool _RunOneTickNonWF(void);													// called by _RunOneTick() to run a tick (nonWF models)
 	
+	EidosValue_SP _EvaluateTickRangeNode(const EidosASTNode *node);					// evaluate a node that represents a tick range expression
+	void EvaluateScriptBlockTickRanges(void);										// evaluate tick range expressions to find when a block is scheduled
 	slim_tick_t FirstTick(void);													// derived from the first tick in which an Eidos block is registered
 	slim_tick_t EstimatedLastTick(void);											// derived from the last tick in which an Eidos block is registered
 	void SimulationHasFinished(void);
@@ -355,6 +358,7 @@ public:
 	virtual EidosValue_SP ExecuteInstanceMethod(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter) override;
 	
 	EidosValue_SP ExecuteMethod_createLogFile(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter);
+	EidosValue_SP ExecuteMethod_estimatedLastTick(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter);
 	EidosValue_SP ExecuteMethod_deregisterScriptBlock(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter);
 	EidosValue_SP ExecuteMethod_genomicElementTypesWithIDs(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter);
 	EidosValue_SP ExecuteMethod_interactionTypesWithIDs(EidosGlobalStringID p_method_id, const std::vector<EidosValue_SP> &p_arguments, EidosInterpreter &p_interpreter);
