@@ -3,7 +3,7 @@
 //  Eidos
 //
 //  Created by Ben Haller on 10/12/20.
-//  Copyright (c) 2020-2023 Philipp Messer.  All rights reserved.
+//  Copyright (c) 2020-2024 Philipp Messer.  All rights reserved.
 //	A product of the Messer Lab, http://messerlab.org/slim/
 //
 
@@ -239,15 +239,13 @@ std::string EidosDictionaryUnretained::Serialization_SLiM(void) const
 		
 		// We want to output our keys in the same order as allKeys, so we just use AllKeys()
 		EidosValue_SP all_keys = AllKeys();
-		EidosValue_String_vector *string_vec = dynamic_cast<EidosValue_String_vector *>(all_keys.get());
+		int all_keys_count = all_keys->Count();
+		const std::string *all_key_strings = all_keys->StringData();
 		
-		if (!string_vec)
-			EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::Serialization_SLiM): (internal error) allKeys did not return a string vector." << EidosTerminate(nullptr);
-		
-		const std::vector<std::string> *all_key_strings = string_vec->StringVector();
-		
-		for (const std::string &key : *all_key_strings)
+		for (int key_index = 0; key_index < all_keys_count; ++key_index)
 		{
+			const std::string key = all_key_strings[key_index];
+			
 			// emit the key; note that we now always quote stringkeys, to distinguish them from the integer-key case easily
 			ss << Eidos_string_escaped(key, EidosStringQuoting::kDoubleQuotes) << "=";
 			
@@ -273,7 +271,7 @@ std::string EidosDictionaryUnretained::Serialization_SLiM(void) const
 		
 		// We want to output our keys in the same order as allKeys, so we just use AllKeys()
 		EidosValue_SP all_keys = AllKeys();
-		EidosValue_Int_vector *integer_vec = dynamic_cast<EidosValue_Int_vector *>(all_keys.get());
+		EidosValue_Int *integer_vec = dynamic_cast<EidosValue_Int *>(all_keys.get());
 		
 		if (!integer_vec)
 			EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::Serialization_SLiM): (internal error) allKeys did not return an integer vector." << EidosTerminate(nullptr);
@@ -281,9 +279,9 @@ std::string EidosDictionaryUnretained::Serialization_SLiM(void) const
 		const int64_t *all_key_integers = integer_vec->data();
 		int all_keys_count = integer_vec->Count();
 		
-		for (int i = 0; i < all_keys_count; ++i)
+		for (int key_index = 0; key_index < all_keys_count; ++key_index)
 		{
-			int64_t key = all_key_integers[i];
+			int64_t key = all_key_integers[key_index];
 			
 			// emit the key
 			ss << key << "=";
@@ -307,6 +305,147 @@ std::string EidosDictionaryUnretained::Serialization_SLiM(void) const
 	return ss.str();
 }
 
+std::string EidosDictionaryUnretained::Serialization_Pretty(int indent_level) const
+{
+	// Loop through our key-value pairs and serialize them
+	if (!state_ptr_)
+		return "{}";
+	
+	std::ostringstream ss;
+	
+	ss << "{";
+	indent_level++;
+	
+	if (KeysAreStrings())
+	{
+		// Serialize a dictionary with string keys
+		const EidosDictionaryHashTable_StringKeys *symbols = DictionarySymbols_StringKeys();
+		
+		// We want to output our keys in the same order as allKeys, so we just use AllKeys()
+		EidosValue_SP all_keys = AllKeys();
+		int all_keys_count = all_keys->Count();
+		const std::string *all_key_strings = all_keys->StringData();
+		
+		for (int key_index = 0; key_index < all_keys_count; ++key_index)
+		{
+			if (key_index == 0)
+				ss << std::endl;
+			ss << std::string(indent_level, '\t');
+			
+			const std::string key = all_key_strings[key_index];
+			
+			// emit the key; note that we now always quote stringkeys, to distinguish them from the integer-key case easily
+			ss << Eidos_string_escaped(key, EidosStringQuoting::kDoubleQuotes) << " = ";
+			
+			// emit the value
+			auto hash_iter = symbols->find(key);
+			
+			if (hash_iter == symbols->end())
+			{
+				// We assume that this is not an internal error, but is instead LogFile with a column that is NA;
+				// it returns all of its column names for AllKeys() even if they have NA as a value, so we play along
+				ss << "NA";
+			}
+			else
+			{
+				EidosValue *value = hash_iter->second.get();
+				
+				if ((value->Type() == EidosValueType::kValueObject) &&
+					((((EidosValue_Object *)value)->Class() == gEidosDictionaryUnretained_Class) || (((EidosValue_Object *)value)->Class() == gEidosDictionaryRetained_Class)))
+				{
+					// Dictionaries get serialized by recursion; subclasses of them do not
+					int value_count = value->Count();
+					
+					for (int value_index = 0; value_index < value_count; ++value_index)
+					{
+						if (value_index > 0)
+							ss << " ";
+						
+						EidosDictionaryUnretained *dict = (EidosDictionaryUnretained *)value->ObjectElementAtIndex_NOCAST(value_index, nullptr);
+						
+						ss << dict->Serialization_Pretty(indent_level);
+					}
+				}
+				else
+				{
+					ss << *value;
+				}
+			}
+			
+			ss << std::endl;
+		}
+	}
+	else	// KeysAreIntegers()
+	{
+		// Serialize a dictionary with integer keys
+		const EidosDictionaryHashTable_IntegerKeys *symbols = DictionarySymbols_IntegerKeys();
+		
+		// We want to output our keys in the same order as allKeys, so we just use AllKeys()
+		EidosValue_SP all_keys = AllKeys();
+		EidosValue_Int *integer_vec = dynamic_cast<EidosValue_Int *>(all_keys.get());
+		
+		if (!integer_vec)
+			EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::Serialization_Pretty): (internal error) allKeys did not return an integer vector." << EidosTerminate(nullptr);
+		
+		const int64_t *all_key_integers = integer_vec->data();
+		int all_keys_count = integer_vec->Count();
+		
+		for (int key_index = 0; key_index < all_keys_count; ++key_index)
+		{
+			if (key_index == 0)
+				ss << std::endl;
+			ss << std::string(indent_level, '\t');
+			
+			int64_t key = all_key_integers[key_index];
+			
+			// emit the key
+			ss << key << " = ";
+			
+			// emit the value
+			auto hash_iter = symbols->find(key);
+			
+			if (hash_iter == symbols->end())
+			{
+				// We assume that this is not an internal error, but is instead LogFile with a column that is NA;
+				// it returns all of its column names for AllKeys() even if they have NA as a value, so we play along
+				ss << "NA;";
+			}
+			else
+			{
+				EidosValue *value = hash_iter->second.get();
+				
+				if ((value->Type() == EidosValueType::kValueObject) &&
+					((((EidosValue_Object *)value)->Class() == gEidosDictionaryUnretained_Class) || (((EidosValue_Object *)value)->Class() == gEidosDictionaryRetained_Class)))
+				{
+					// Dictionaries get serialized by recursion; subclasses of them do not
+					int value_count = value->Count();
+					
+					for (int value_index = 0; value_index < value_count; ++value_index)
+					{
+						if (value_index > 0)
+							ss << " ";
+						
+						EidosDictionaryUnretained *dict = (EidosDictionaryUnretained *)value->ObjectElementAtIndex_NOCAST(value_index, nullptr);
+						
+						ss << dict->Serialization_Pretty(indent_level);
+					}
+				}
+				else
+				{
+					ss << *value;
+				}
+			}
+			
+			ss << std::endl;
+		}
+	}
+	
+	indent_level--;
+	ss << std::string(indent_level, '\t') << "}";
+	
+	return ss.str();
+}
+
 EidosValue_SP EidosDictionaryUnretained::Serialization_CSV(const std::string &p_delimiter) const
 {
 	if (!state_ptr_)
@@ -342,7 +481,7 @@ EidosValue_SP EidosDictionaryUnretained::Serialization_CSV(const std::string &p_
 	
 	// Make a string vector big enough for everything
 	int result_count = longest_col + 1;		// room for the header
-	EidosValue_String_vector *string_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector())->Reserve(result_count);
+	EidosValue_String *string_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_String())->Reserve(result_count);
 	EidosValue_SP result_SP(string_result);
 	
 	// Generate the header
@@ -411,19 +550,19 @@ EidosValue_SP EidosDictionaryUnretained::Serialization_CSV(const std::string &p_
 					case EidosValueType::kValueObject:
 						EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::Serialization_CSV): cannot serialize values of type object to CSV/TSV." << EidosTerminate(nullptr);
 						
-					case EidosValueType::kValueLogical: ss << (value->LogicalAtIndex(row_index, nullptr) ? "TRUE" : "FALSE"); break;
-					case EidosValueType::kValueInt: ss << value->IntAtIndex(row_index, nullptr); break;
+					case EidosValueType::kValueLogical: ss << (value->LogicalAtIndex_NOCAST(row_index, nullptr) ? "TRUE" : "FALSE"); break;
+					case EidosValueType::kValueInt: ss << value->IntAtIndex_NOCAST(row_index, nullptr); break;
 					case EidosValueType::kValueFloat:
 					{
 						int old_precision = gEidosFloatOutputPrecision;
 						gEidosFloatOutputPrecision = EIDOS_DBL_DIGS - 2;	// try to avoid ugly values that exhibit the precision limits
-						ss << EidosStringForFloat(value->FloatAtIndex(row_index, nullptr));
+						ss << EidosStringForFloat(value->FloatAtIndex_NOCAST(row_index, nullptr));
 						gEidosFloatOutputPrecision = old_precision;
 						break;
 					}
 					case EidosValueType::kValueString:
 					{
-						ss << Eidos_string_escaped_CSV(value->StringAtIndex(row_index, nullptr));
+						ss << Eidos_string_escaped_CSV(value->StringAtIndex_NOCAST(row_index, nullptr));
 						break;
 					}
 				}
@@ -446,15 +585,13 @@ nlohmann::json EidosDictionaryUnretained::JSONRepresentation(void) const
 	
 	// We want to output our keys in the same order as allKeys, so we just use AllKeys()
 	EidosValue_SP all_keys = AllKeys();
-	EidosValue_String_vector *string_vec = dynamic_cast<EidosValue_String_vector *>(all_keys.get());
+	int all_keys_count = all_keys->Count();
+	const std::string *all_key_strings = all_keys->StringData();
 	
-	if (!string_vec)
-		EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::JSONRepresentation): (internal error) allKeys did not return a string vector." << EidosTerminate(nullptr);
-	
-	const std::vector<std::string> *all_key_strings = string_vec->StringVector();
-	
-	for (const std::string &key : *all_key_strings)
+	for (int key_index = 0; key_index < all_keys_count; ++key_index)
 	{
+		const std::string key = all_key_strings[key_index];
+		
 		// get the value
 		auto hash_iter = symbols->find(key);
 		
@@ -609,7 +746,7 @@ EidosValue_SP EidosDictionaryUnretained::AllKeys(void) const
 		if (key_count == 0)
 			return gStaticEidosValue_String_ZeroVec;
 		
-		EidosValue_String_vector *string_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector())->Reserve(key_count);
+		EidosValue_String *string_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_String())->Reserve(key_count);
 		
 		for (const std::string &key : keys)
 			string_result->PushString(key);
@@ -624,7 +761,7 @@ EidosValue_SP EidosDictionaryUnretained::AllKeys(void) const
 		if (key_count == 0)
 			return gStaticEidosValue_Integer_ZeroVec;
 		
-		EidosValue_Int_vector *integer_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->reserve(key_count);
+		EidosValue_Int *integer_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->reserve(key_count);
 		
 		for (int64_t key : keys)
 			integer_result->push_int_no_check(key);
@@ -866,7 +1003,7 @@ void EidosDictionaryUnretained::AddJSONFrom(nlohmann::json &json)
 				if (value.is_null())
 				{
 					EidosDictionaryRetained *dictionary = new EidosDictionaryRetained();
-					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object_singleton(dictionary, gEidosDictionaryRetained_Class));
+					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(dictionary, gEidosDictionaryRetained_Class));
 					dictionary->Release();		// now retained by state_ptr->dictionary_symbols_
 				}
 				else if (value.is_boolean())
@@ -877,23 +1014,23 @@ void EidosDictionaryUnretained::AddJSONFrom(nlohmann::json &json)
 				else if (value.is_string())
 				{
 					const std::string &string_value = value;
-					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(string_value));
+					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(string_value));
 				}
 				else if (value.is_number_integer() || value.is_number_unsigned())
 				{
 					int64_t int_value = value;
-					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int_singleton(int_value));
+					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Int(int_value));
 				}
 				else if (value.is_number_float())
 				{
 					double float_value = value;
-					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float_singleton(float_value));
+					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Float(float_value));
 				}
 				else if (value.is_object())
 				{
 					EidosDictionaryRetained *dictionary = new EidosDictionaryRetained();
 					dictionary->AddJSONFrom(value);
-					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object_singleton(dictionary, gEidosDictionaryRetained_Class));
+					state_ptr->dictionary_symbols_[key] = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(dictionary, gEidosDictionaryRetained_Class));
 					dictionary->Release();		// now retained by state_ptr->dictionary_symbols_
 				}
 				else if (value.is_array())
@@ -931,7 +1068,7 @@ void EidosDictionaryUnretained::AddJSONFrom(nlohmann::json &json)
 						// ok, all elements are of a single type, so let's create a vector of that type from the values in the array
 						if (array_type == nlohmann::json::value_t::number_integer)
 						{
-							EidosValue_Int_vector *int_value = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->resize_no_initialize(array_count);
+							EidosValue_Int *int_value = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->resize_no_initialize(array_count);
 							
 							for (size_t element_index = 0; element_index < array_count; ++element_index)
 							{
@@ -943,7 +1080,7 @@ void EidosDictionaryUnretained::AddJSONFrom(nlohmann::json &json)
 						}
 						else if (array_type == nlohmann::json::value_t::number_float)
 						{
-							EidosValue_Float_vector *float_value = (new (gEidosValuePool->AllocateChunk()) EidosValue_Float_vector())->resize_no_initialize(array_count);
+							EidosValue_Float *float_value = (new (gEidosValuePool->AllocateChunk()) EidosValue_Float())->resize_no_initialize(array_count);
 							
 							for (size_t element_index = 0; element_index < array_count; ++element_index)
 							{
@@ -967,7 +1104,7 @@ void EidosDictionaryUnretained::AddJSONFrom(nlohmann::json &json)
 						}
 						else if (array_type == nlohmann::json::value_t::string)
 						{
-							EidosValue_String_vector *string_value = (new (gEidosValuePool->AllocateChunk()) EidosValue_String_vector())->Reserve((int)array_count);
+							EidosValue_String *string_value = (new (gEidosValuePool->AllocateChunk()) EidosValue_String())->Reserve((int)array_count);
 							
 							for (size_t element_index = 0; element_index < array_count; ++element_index)
 							{
@@ -979,7 +1116,7 @@ void EidosDictionaryUnretained::AddJSONFrom(nlohmann::json &json)
 						}
 						else if (array_type == nlohmann::json::value_t::object)
 						{
-							EidosValue_Object_vector *object_value = new (gEidosValuePool->AllocateChunk()) EidosValue_Object_vector(gEidosDictionaryRetained_Class);
+							EidosValue_Object *object_value = new (gEidosValuePool->AllocateChunk()) EidosValue_Object(gEidosDictionaryRetained_Class);
 							
 							for (size_t element_index = 0; element_index < array_count; ++element_index)
 							{
@@ -1085,7 +1222,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_addKeysAndValuesFrom(Eido
 	// Check that source is a subclass of EidosDictionaryUnretained.  We do this check here because we want to avoid making
 	// EidosDictionaryUnretained visible in the public API; we want to pretend that there is just one class, Dictionary.
 	// I'm not sure whether that's going to be right in the long term, but I want to keep my options open for now.
-	EidosDictionaryUnretained *source = dynamic_cast<EidosDictionaryUnretained *>(source_value->ObjectElementAtIndex(0, nullptr));
+	EidosDictionaryUnretained *source = dynamic_cast<EidosDictionaryUnretained *>(source_value->ObjectElementAtIndex_NOCAST(0, nullptr));
 	
 	if (!source)
 		EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::ExecuteMethod_addKeysAndValuesFrom): addKeysAndValuesFrom() can only take values from a Dictionary or a subclass of Dictionary." << EidosTerminate(nullptr);
@@ -1108,7 +1245,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_appendKeysAndValuesFrom(E
 	// Loop through elements in source and handle them sequentially
 	for (int value_index = 0; value_index < source_count; ++value_index)
 	{
-		EidosDictionaryUnretained *source = dynamic_cast<EidosDictionaryUnretained *>(source_value->ObjectElementAtIndex(value_index, nullptr));
+		EidosDictionaryUnretained *source = dynamic_cast<EidosDictionaryUnretained *>(source_value->ObjectElementAtIndex_NOCAST(value_index, nullptr));
 		
 		// Check that source is a subclass of EidosDictionaryUnretained.  We do this check here because we want to avoid making
 		// EidosDictionaryUnretained visible in the public API; we want to pretend that there is just one class, Dictionary.
@@ -1142,7 +1279,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_compactIndices(EidosGloba
 {
 #pragma unused (p_method_id, p_interpreter)
 	EidosValue *preserveOrder_value = p_arguments[0].get();
-	bool preserveOrder = preserveOrder_value->LogicalAtIndex(0, nullptr);
+	bool preserveOrder = preserveOrder_value->LogicalAtIndex_NOCAST(0, nullptr);
 	
 	if (!KeysAreIntegers())
 		EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::ExecuteMethod_compactIndices): compactIndices() can only be called on a dictionary that uses integer keys." << EidosTerminate(nullptr);
@@ -1153,7 +1290,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_compactIndices(EidosGloba
 	EidosDictionaryState_IntegerKeys *state_ptr = (EidosDictionaryState_IntegerKeys *)state_ptr_;
 	EidosDictionaryHashTable_IntegerKeys &symbols = state_ptr->dictionary_symbols_;
 	size_t key_count = symbols.size();
-	EidosValue_Int_vector *integer_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int_vector())->reserve(key_count);
+	EidosValue_Int *integer_result = (new (gEidosValuePool->AllocateChunk()) EidosValue_Int())->reserve(key_count);
 	std::vector<EidosValue_SP> compacted_values;
 	
 	if (preserveOrder)
@@ -1222,7 +1359,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_getRowValues(EidosGlobalS
 	EidosValue_SP result_SP(nullptr);
 	
 	EidosDictionaryRetained *objectElement = new EidosDictionaryRetained();
-	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object_singleton(objectElement, gEidosDictionaryRetained_Class));
+	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(objectElement, gEidosDictionaryRetained_Class));
 	
 	// With no columns, the indices don't matter, and the result is a new empty dictionary
 	if (KeyCount() == 0)
@@ -1239,7 +1376,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_getRowValues(EidosGlobalS
 	{
 		const EidosDictionaryHashTable_StringKeys *symbols = DictionarySymbols_StringKeys();
 		const std::vector<std::string> keys = SortedKeys_StringKeys();
-		bool drop = drop_value->LogicalAtIndex(0, nullptr);
+		bool drop = drop_value->LogicalAtIndex_NOCAST(0, nullptr);
 		
 		for (const std::string &key : keys)
 		{
@@ -1258,7 +1395,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_getRowValues(EidosGlobalS
 	{
 		const EidosDictionaryHashTable_IntegerKeys *symbols = DictionarySymbols_IntegerKeys();
 		const std::vector<int64_t> keys = SortedKeys_IntegerKeys();
-		bool drop = drop_value->LogicalAtIndex(0, nullptr);
+		bool drop = drop_value->LogicalAtIndex_NOCAST(0, nullptr);
 		
 		for (int64_t key : keys)
 		{
@@ -1298,7 +1435,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_getValue(EidosGlobalStrin
 		if (key_value->Type() != EidosValueType::kValueString)
 			EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::ExecuteMethod_getValue): an integer key was supplied to getValue(), but the target dictionary uses string keys." << EidosTerminate(nullptr);
 		
-		const std::string &key = ((EidosValue_String *)key_value)->StringRefAtIndex(0, nullptr);
+		const std::string &key = ((EidosValue_String *)key_value)->StringRefAtIndex_NOCAST(0, nullptr);
 		const EidosDictionaryHashTable_StringKeys *symbols = DictionarySymbols_StringKeys();
 		
 		auto found_iter = symbols->find(key);
@@ -1313,7 +1450,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_getValue(EidosGlobalStrin
 		if (key_value->Type() != EidosValueType::kValueInt)
 			EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::ExecuteMethod_getValue): a string key was supplied to getValue(), but the target dictionary uses integer keys." << EidosTerminate(nullptr);
 		
-		int64_t key = ((EidosValue_Int *)key_value)->IntAtIndex(0, nullptr);
+		int64_t key = ((EidosValue_Int *)key_value)->IntAtIndex_NOCAST(0, nullptr);
 		const EidosDictionaryHashTable_IntegerKeys *symbols = DictionarySymbols_IntegerKeys();
 		
 		auto found_iter = symbols->find(key);
@@ -1331,7 +1468,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_identicalContents(EidosGl
 {
 #pragma unused (p_method_id, p_arguments, p_interpreter)
 	EidosValue *x_value = p_arguments[0].get();
-	EidosObject *x_object = x_value->ObjectElementAtIndex(0, nullptr);
+	EidosObject *x_object = x_value->ObjectElementAtIndex_NOCAST(0, nullptr);
 	EidosDictionaryUnretained *x_dict = dynamic_cast<EidosDictionaryUnretained *>(x_object);
 	
 	if (!x_dict)
@@ -1433,13 +1570,13 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_Accelerated_setValue(Eido
 		// targets.  That made me nervous, and was hard to reconcile with DataFrame, so I removed it.
 		if (key_value->Type() == EidosValueType::kValueString)
 		{
-			const std::string &key = ((EidosValue_String *)key_value)->StringRefAtIndex(0, nullptr);
+			const std::string &key = ((EidosValue_String *)key_value)->StringRefAtIndex_NOCAST(0, nullptr);
 			
 			element->SetKeyValue_StringKeys(key, value);
 		}
 		else
 		{
-			int64_t key = ((EidosValue_Int *)key_value)->IntAtIndex(0, nullptr);
+			int64_t key = ((EidosValue_Int *)key_value)->IntAtIndex_NOCAST(0, nullptr);
 			
 			element->SetKeyValue_IntegerKeys(key, value);
 		}
@@ -1456,18 +1593,22 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_serialize(EidosGlobalStri
 {
 #pragma unused (p_method_id, p_interpreter)
 	EidosValue_String *string_value = (EidosValue_String *)p_arguments[0].get();
-	const std::string &format_name = string_value->StringRefAtIndex(0, nullptr);
+	const std::string &format_name = string_value->StringRefAtIndex_NOCAST(0, nullptr);
 	
 	if (format_name == "slim")
 	{
-		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(Serialization_SLiM()));
+		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(Serialization_SLiM()));
+	}
+	else if (format_name == "pretty")
+	{
+		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(Serialization_Pretty(0)));
 	}
 	else if (format_name == "json")
 	{
 		nlohmann::json json_rep = JSONRepresentation();
 		std::string json_string = json_rep.dump();
 		
-		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton(json_string));
+		return EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String(json_string));
 	}
 	else if (format_name == "csv")
 	{
@@ -1479,7 +1620,7 @@ EidosValue_SP EidosDictionaryUnretained::ExecuteMethod_serialize(EidosGlobalStri
 	}
 	else
 	{
-		EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::ExecuteMethod_serialize): serialize() does not recognize the format '" << format_name << "'; it should be 'slim', 'json', 'csv', or 'tsv'." << EidosTerminate(nullptr);
+		EIDOS_TERMINATION << "ERROR (EidosDictionaryUnretained::ExecuteMethod_serialize): serialize() does not recognize the format '" << format_name << "'; it should be 'slim', 'pretty', 'json', 'csv', or 'tsv'." << EidosTerminate(nullptr);
 	}
 }
 
@@ -1529,7 +1670,7 @@ const std::vector<EidosMethodSignature_CSP> *EidosDictionaryUnretained_Class::Me
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_getRowValues, kEidosValueMaskObject | kEidosValueMaskSingleton, gEidosDictionaryRetained_Class))->AddArg(kEidosValueMaskLogical | kEidosValueMaskInt, "index", nullptr)->AddLogical_OS("drop", gStaticEidosValue_LogicalF));
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_getValue, kEidosValueMaskAny))->AddArg(kEidosValueMaskInt | kEidosValueMaskString | kEidosValueMaskSingleton, "key", nullptr));
 		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_identicalContents, kEidosValueMaskLogical | kEidosValueMaskSingleton))->AddObject_S("x", nullptr));
-		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_serialize, kEidosValueMaskString))->AddString_OS("format", EidosValue_String_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String_singleton("slim"))));
+		methods->emplace_back((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_serialize, kEidosValueMaskString))->AddString_OS("format", EidosValue_String_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_String("slim"))));
 		methods->emplace_back(((EidosInstanceMethodSignature *)(new EidosInstanceMethodSignature(gEidosStr_setValue, kEidosValueMaskVOID))->AddArg(kEidosValueMaskInt | kEidosValueMaskString | kEidosValueMaskSingleton, "key", nullptr)->AddAny("value"))->DeclareAcceleratedImp(EidosDictionaryUnretained::ExecuteMethod_Accelerated_setValue));
 		
 		std::sort(methods->begin(), methods->end(), CompareEidosCallSignatures);
@@ -1575,16 +1716,26 @@ void EidosDictionaryRetained::ConstructFromEidos(const std::vector<EidosValue_SP
 	}
 	else if (p_arguments.size() == 1)
 	{
-		// one singleton argument; multiple overloaded meanings
+		// one argument; multiple overloaded meanings
 		EidosValue *source_value = p_arguments[0].get();
-		
-		if (source_value->Count() != 1)
-			EIDOS_TERMINATION << "ERROR (" << p_caller_name << "): " << p_constructor_name << "(x) requires that x be a singleton (Dictionary, Dictionary subclass, or JSON string)." << EidosTerminate(nullptr);
+		int source_count = source_value->Count();
 		
 		if (source_value->Type() == EidosValueType::kValueString)
 		{
-			// Construct from a JSON string
-			std::string json_string = source_value->StringAtIndex(0, nullptr);
+			// Construct from a JSON string; beginning in SLiM 4.2 this is allowed to be a string vector,
+			// as returned by readFile().  We just paste the lines back together with newlines.  Note that
+			// if the user tries to put a newline inside a string value, that is actually not legal JSON
+			// (see https://stackoverflow.com/a/16690186/2752221), and nlohmann will error below.
+			std::string json_string;
+			
+			for (int source_index = 0; source_index < source_count; ++source_index)
+			{
+				if (source_index > 0)
+					json_string.append("\n");
+				
+				json_string.append(source_value->StringAtIndex_NOCAST(source_index, nullptr));
+			}
+			
 			nlohmann::json json_rep;
 			
 			try {
@@ -1597,11 +1748,14 @@ void EidosDictionaryRetained::ConstructFromEidos(const std::vector<EidosValue_SP
 		}
 		else
 		{
-			// Construct from a Dictionary or Dictionary subclass
-			EidosDictionaryUnretained *source = (source_value->Type() != EidosValueType::kValueObject) ? nullptr : dynamic_cast<EidosDictionaryUnretained *>(source_value->ObjectElementAtIndex(0, nullptr));
+			// Construct from a singleton Dictionary or Dictionary subclass
+			if (source_count != 1)
+				EIDOS_TERMINATION << "ERROR (" << p_caller_name << "): " << p_constructor_name << "(x) requires that x be a singleton Dictionary (or a singleton subclass of Dictionary), or a string vector." << EidosTerminate(nullptr);
+			
+			EidosDictionaryUnretained *source = (source_value->Type() != EidosValueType::kValueObject) ? nullptr : dynamic_cast<EidosDictionaryUnretained *>(source_value->ObjectElementAtIndex_NOCAST(0, nullptr));
 		
 			if (!source)
-				EIDOS_TERMINATION << "ERROR (" << p_caller_name << "): " << p_constructor_name << "(x) requires that x be a singleton Dictionary (or a singleton subclass of Dictionary)." << EidosTerminate(nullptr);
+				EIDOS_TERMINATION << "ERROR (" << p_caller_name << "): " << p_constructor_name << "(x) requires that x be a singleton Dictionary (or a singleton subclass of Dictionary), or a string vector." << EidosTerminate(nullptr);
 		
 			AddKeysAndValuesFrom(source);
 		}
@@ -1628,13 +1782,13 @@ void EidosDictionaryRetained::ConstructFromEidos(const std::vector<EidosValue_SP
 			{
 				EidosValue_String *key_string_value = (EidosValue_String *)key;
 				
-				SetKeyValue_StringKeys(key_string_value->StringRefAtIndex(0, nullptr), value);
+				SetKeyValue_StringKeys(key_string_value->StringRefAtIndex_NOCAST(0, nullptr), value);
 			}
 			else if (key->Type() == EidosValueType::kValueInt)
 			{
 				EidosValue_Int *key_integer_value = (EidosValue_Int *)key;
 				
-				SetKeyValue_IntegerKeys(key_integer_value->IntAtIndex(0, nullptr), value);
+				SetKeyValue_IntegerKeys(key_integer_value->IntAtIndex_NOCAST(0, nullptr), value);
 			}
 			else
 			{
@@ -1652,7 +1806,7 @@ static EidosValue_SP Eidos_Instantiate_EidosDictionaryRetained(const std::vector
 	EidosValue_SP result_SP(nullptr);
 	
 	EidosDictionaryRetained *objectElement = new EidosDictionaryRetained();
-	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object_singleton(objectElement, gEidosDictionaryRetained_Class));
+	result_SP = EidosValue_SP(new (gEidosValuePool->AllocateChunk()) EidosValue_Object(objectElement, gEidosDictionaryRetained_Class));
 	
 	// objectElement is now retained by result_SP, so we can release it
 	objectElement->Release();
